@@ -8,23 +8,19 @@
 #define RABIT_SERIALIZABLE_H_
 #include <vector>
 #include <string>
-#include "./internal/utils.h"
-
-#ifndef DMLC_IO_H_
-#include "../../dmlc-core/include/dmlc/io.h"
-#endif  // DMLC_IO_H_
+#include "rabit/internal/utils.h"
 
 namespace rabit {
 /*!
  * \brief defines stream used in rabit
  * see definition of Stream in dmlc/io.h
  */
-typedef dmlc::Stream Stream;
+using Stream = dmlc::Stream ;
 /*!
  * \brief defines serializable objects used in rabit
  * see definition of Serializable in dmlc/io.h
  */
-typedef dmlc::Serializable Serializable;
+using Serializable = dmlc::Serializable;
 
 }  // namespace rabit
 #endif  // RABIT_SERIALIZABLE_H_
