@@ -18,8 +18,8 @@
 #include "xgboost/base.h"
 #include "xgboost/data.h"
 #include "xgboost/span.h"
-#include "xgboost/c_api.h"
 
+#include "array_interface.h"
 #include "../c_api/c_api_error.h"
 
 namespace xgboost {
@@ -136,8 +136,7 @@ class CSRAdapterBatch : public detail::NoMetaInfo {
     const float* values_;
   };
   CSRAdapterBatch(const size_t* row_ptr, const unsigned* feature_idx,
-                  const float* values, size_t num_rows, size_t num_elements,
-                  size_t num_features)
+                  const float* values, size_t num_rows, size_t, size_t)
       : row_ptr_(row_ptr),
         feature_idx_(feature_idx),
         values_(values),
@@ -496,6 +495,7 @@ class FileAdapter : dmlc::DataIter<FileAdapterBatch> {
 
 /*! \brief Data iterator that takes callback to return data, used in JVM package for
  *  accepting data iterator. */
+template <typename DataIterHandle, typename XGBCallbackDataIterNext, typename XGBoostBatchCSR>
 class IteratorAdapter : public dmlc::DataIter<FileAdapterBatch> {
  public:
   IteratorAdapter(DataIterHandle data_handle,
