@@ -22,33 +22,25 @@ namespace data {
 // Used for external memory.
 class SparsePageDMatrix : public DMatrix {
  public:
-  explicit SparsePageDMatrix(std::unique_ptr<DataSource<SparsePage>>&& source,
-                             std::string cache_info)
-      : row_source_(std::move(source)), cache_info_(std::move(cache_info)) {}
-
   template <typename AdapterT>
   explicit SparsePageDMatrix(AdapterT* adapter, float missing, int nthread,
                              const std::string& cache_prefix,
                              size_t page_size = kPageSize)
       : cache_info_(std::move(cache_prefix)) {
-    if (!data::SparsePageSource<SparsePage>::CacheExist(cache_prefix,
-                                                        ".row.page")) {
-      data::SparsePageSource<SparsePage>::CreateRowPage(
-          adapter, missing, nthread, cache_prefix, page_size);
-    }
-    row_source_.reset(
-        new data::SparsePageSource<SparsePage>(cache_prefix, ".row.page"));
+    row_source_.reset(new data::SparsePageSource(adapter, missing, nthread,
+                                                 cache_prefix, page_size));
   }
-    // Set number of threads but keep old value so we can reset it after
   ~SparsePageDMatrix() override = default;
 
   MetaInfo& Info() override;
 
   const MetaInfo& Info() const override;
 
-  float GetColDensity(size_t cidx) override;
-
-  bool SingleColBlock() const override;
+  bool SingleColBlock() const override { return false; }
+  DMatrix *Slice(common::Span<int32_t const>) override {
+    LOG(FATAL) << "Slicing DMatrix is not supported for external memory.";
+    return nullptr;
+  }
 
  private:
   BatchSet<SparsePage> GetRowBatches() override;
@@ -57,9 +49,9 @@ class SparsePageDMatrix : public DMatrix {
   BatchSet<EllpackPage> GetEllpackBatches(const BatchParam& param) override;
 
   // source data pointers.
-  std::unique_ptr<DataSource<SparsePage>> row_source_;
-  std::unique_ptr<SparsePageSource<CSCPage>> column_source_;
-  std::unique_ptr<SparsePageSource<SortedCSCPage>> sorted_column_source_;
+  std::unique_ptr<SparsePageSource> row_source_;
+  std::unique_ptr<CSCPageSource> column_source_;
+  std::unique_ptr<SortedCSCPageSource> sorted_column_source_;
   std::unique_ptr<EllpackPageSource> ellpack_source_;
   // saved batch param
   BatchParam batch_param_;
@@ -67,6 +59,13 @@ class SparsePageDMatrix : public DMatrix {
   std::string cache_info_;
   // Store column densities to avoid recalculating
   std::vector<float> col_density_;
+
+  bool EllpackExists() const override {
+    return static_cast<bool>(ellpack_source_);
+  }
+  bool SparsePageExists() const override {
+    return static_cast<bool>(row_source_);
+  }
 };
 }  // namespace data
 }  // namespace xgboost
