@@ -113,7 +113,7 @@ class GBLinear : public GradientBooster {
 
   void DoBoost(DMatrix *p_fmat,
                HostDeviceVector<GradientPair> *in_gpair,
-               PredictionCacheEntry* predt) override {
+               PredictionCacheEntry*) override {
     monitor_.Start("DoBoost");
 
     model_.LazyInitModel();
@@ -128,8 +128,7 @@ class GBLinear : public GradientBooster {
 
   void PredictBatch(DMatrix *p_fmat,
                     PredictionCacheEntry *predts,
-                    bool training,
-                    unsigned ntree_limit) override {
+                    bool, unsigned ntree_limit) override {
     monitor_.Start("PredictBatch");
     auto* out_preds = &predts->predictions;
     CHECK_EQ(ntree_limit, 0U)
@@ -140,7 +139,7 @@ class GBLinear : public GradientBooster {
   // add base margin
   void PredictInstance(const SparsePage::Inst &inst,
                        std::vector<bst_float> *out_preds,
-                       unsigned ntree_limit) override {
+                       unsigned) override {
     const int ngroup = model_.learner_model_param->num_output_group;
     for (int gid = 0; gid < ngroup; ++gid) {
       this->Pred(inst, dmlc::BeginPtr(*out_preds), gid,
@@ -148,16 +147,13 @@ class GBLinear : public GradientBooster {
     }
   }
 
-  void PredictLeaf(DMatrix *p_fmat,
-                   std::vector<bst_float> *out_preds,
-                   unsigned ntree_limit) override {
+  void PredictLeaf(DMatrix *, HostDeviceVector<bst_float> *, unsigned) override {
     LOG(FATAL) << "gblinear does not support prediction of leaf index";
   }
 
   void PredictContribution(DMatrix* p_fmat,
-                           std::vector<bst_float>* out_contribs,
-                           unsigned ntree_limit, bool approximate, int condition = 0,
-                           unsigned condition_feature = 0) override {
+                           HostDeviceVector<bst_float>* out_contribs,
+                           unsigned ntree_limit, bool, int, unsigned) override {
     model_.LazyInitModel();
     CHECK_EQ(ntree_limit, 0U)
         << "GBLinear::PredictContribution: ntrees is only valid for gbtree predictor";
@@ -165,7 +161,7 @@ class GBLinear : public GradientBooster {
     const int ngroup = model_.learner_model_param->num_output_group;
     const size_t ncolumns = model_.learner_model_param->num_feature + 1;
     // allocate space for (#features + bias) times #groups times #rows
-    std::vector<bst_float>& contribs = *out_contribs;
+    std::vector<bst_float>& contribs = out_contribs->HostVector();
     contribs.resize(p_fmat->Info().num_row_ * ncolumns * ngroup);
     // make sure contributions is zeroed, we could be reusing a previously allocated one
     std::fill(contribs.begin(), contribs.end(), 0);
@@ -195,9 +191,9 @@ class GBLinear : public GradientBooster {
   }
 
   void PredictInteractionContributions(DMatrix* p_fmat,
-                                       std::vector<bst_float>* out_contribs,
-                                       unsigned ntree_limit, bool approximate) override {
-    std::vector<bst_float>& contribs = *out_contribs;
+                                       HostDeviceVector<bst_float>* out_contribs,
+                                       unsigned, bool) override {
+    std::vector<bst_float>& contribs = out_contribs->HostVector();
 
     // linear models have no interaction effects
     const size_t nelements = model_.learner_model_param->num_feature *
@@ -231,7 +227,8 @@ class GBLinear : public GradientBooster {
     // start collecting the prediction
     const int ngroup = model_.learner_model_param->num_output_group;
     preds.resize(p_fmat->Info().num_row_ * ngroup);
-    for (const auto &batch : p_fmat->GetBatches<SparsePage>()) {
+    for (const auto &page : p_fmat->GetBatches<SparsePage>()) {
+      auto const& batch = page.GetView();
       // output convention: nrow * k, where nrow is number of rows
       // k is number of group
       // parallel over local batch
@@ -241,7 +238,7 @@ class GBLinear : public GradientBooster {
       }
 #pragma omp parallel for schedule(static)
       for (omp_ulong i = 0; i < nsize; ++i) {
-        const size_t ridx = batch.base_rowid + i;
+        const size_t ridx = page.base_rowid + i;
         // loop over output groups
         for (int gid = 0; gid < ngroup; ++gid) {
           bst_float margin =
