@@ -17,7 +17,6 @@
 #include "param.h"
 #include "constraints.h"
 #include "../common/random.h"
-#include "../common/bitmap.h"
 #include "split_evaluator.h"
 
 namespace xgboost {
@@ -56,27 +55,41 @@ class ColMaker: public TreeUpdater {
   void Configure(const Args& args) override {
     param_.UpdateAllowUnknown(args);
     colmaker_param_.UpdateAllowUnknown(args);
-    if (!spliteval_) {
-      spliteval_.reset(SplitEvaluator::Create(param_.split_evaluator));
-    }
-    spliteval_->Init(&param_);
   }
 
   void LoadConfig(Json const& in) override {
     auto const& config = get<Object const>(in);
-    fromJson(config.at("train_param"), &this->param_);
-    fromJson(config.at("colmaker_train_param"), &this->colmaker_param_);
+    FromJson(config.at("train_param"), &this->param_);
+    FromJson(config.at("colmaker_train_param"), &this->colmaker_param_);
   }
   void SaveConfig(Json* p_out) const override {
     auto& out = *p_out;
-    out["train_param"] = toJson(param_);
-    out["colmaker_train_param"] = toJson(colmaker_param_);
+    out["train_param"] = ToJson(param_);
+    out["colmaker_train_param"] = ToJson(colmaker_param_);
   }
 
   char const* Name() const override {
     return "grow_colmaker";
   }
 
+  void LazyGetColumnDensity(DMatrix *dmat) {
+    // Finds densities if we don't already have them
+    if (column_densities_.empty()) {
+      std::vector<size_t> column_size(dmat->Info().num_col_);
+      for (const auto &batch : dmat->GetBatches<SortedCSCPage>()) {
+        for (auto i = 0u; i < batch.Size(); i++) {
+          column_size[i] += batch[i].size();
+        }
+      }
+      column_densities_.resize(column_size.size());
+      for (auto i = 0u; i < column_densities_.size(); i++) {
+        size_t nmiss = dmat->Info().num_row_ - column_size[i];
+        column_densities_[i] =
+            1.0f - (static_cast<float>(nmiss)) / dmat->Info().num_row_;
+      }
+    }
+  }
+
   void Update(HostDeviceVector<GradientPair> *gpair,
               DMatrix* dmat,
               const std::vector<RegTree*> &trees) override {
@@ -84,6 +97,7 @@ class ColMaker: public TreeUpdater {
       LOG(FATAL) << "Updater `grow_colmaker` or `exact` tree method doesn't "
                     "support distributed training.";
     }
+    this->LazyGetColumnDensity(dmat);
     // rescale learning rate according to size of trees
     float lr = param_.learning_rate;
     param_.learning_rate = lr / trees.size();
@@ -93,8 +107,7 @@ class ColMaker: public TreeUpdater {
       Builder builder(
         param_,
         colmaker_param_,
-        std::unique_ptr<SplitEvaluator>(spliteval_->GetHostClone()),
-        interaction_constraints_);
+        interaction_constraints_, column_densities_);
       builder.Update(gpair->ConstHostVector(), dmat, tree);
     }
     param_.learning_rate = lr;
@@ -105,7 +118,7 @@ class ColMaker: public TreeUpdater {
   TrainParam param_;
   ColMakerTrainParam colmaker_param_;
   // SplitEvaluator that will be cloned for each Builder
-  std::unique_ptr<SplitEvaluator> spliteval_;
+  std::vector<float> column_densities_;
 
   FeatureInteractionConstraintHost interaction_constraints_;
   // data structure
@@ -114,23 +127,23 @@ class ColMaker: public TreeUpdater {
     /*! \brief statistics of data */
     GradStats stats;
     /*! \brief last feature value scanned */
-    bst_float last_fvalue;
+    bst_float last_fvalue { 0 };
     /*! \brief current best solution */
     SplitEntry best;
     // constructor
-    ThreadEntry() : last_fvalue{0} {}
+    ThreadEntry() = default;
   };
   struct NodeEntry {
     /*! \brief statics for node entry */
     GradStats stats;
     /*! \brief loss of this node, without split */
-    bst_float root_gain;
+    bst_float root_gain { 0.0f };
     /*! \brief weight calculated related to current data */
-    bst_float weight;
+    bst_float weight { 0.0f };
     /*! \brief current best solution */
     SplitEntry best;
     // constructor
-    NodeEntry() : root_gain{0.0f}, weight{0.0f} {}
+    NodeEntry() = default;
   };
   // actual builder that runs the algorithm
   class Builder {
@@ -138,18 +151,19 @@ class ColMaker: public TreeUpdater {
     // constructor
     explicit Builder(const TrainParam& param,
                      const ColMakerTrainParam& colmaker_train_param,
-                     std::unique_ptr<SplitEvaluator> spliteval,
-                     FeatureInteractionConstraintHost _interaction_constraints)
+                     FeatureInteractionConstraintHost _interaction_constraints,
+                     const std::vector<float> &column_densities)
         : param_(param), colmaker_train_param_{colmaker_train_param},
           nthread_(omp_get_max_threads()),
-          spliteval_(std::move(spliteval)),
-          interaction_constraints_{std::move(_interaction_constraints)} {}
+          tree_evaluator_(param_, column_densities.size(), GenericParameter::kCpuId),
+          interaction_constraints_{std::move(_interaction_constraints)},
+          column_densities_(column_densities) {}
     // update one tree, growing
     virtual void Update(const std::vector<GradientPair>& gpair,
                         DMatrix* p_fmat,
                         RegTree* p_tree) {
       std::vector<int> newnodes;
-      this->InitData(gpair, *p_fmat, *p_tree);
+      this->InitData(gpair, *p_fmat);
       this->InitNewNode(qexpand_, gpair, *p_fmat, *p_tree);
       for (int depth = 0; depth < param_.max_depth; ++depth) {
         this->FindSplit(depth, qexpand_, gpair, p_fmat, p_tree);
@@ -162,12 +176,9 @@ class ColMaker: public TreeUpdater {
           }
           int cleft = (*p_tree)[nid].LeftChild();
           int cright = (*p_tree)[nid].RightChild();
-          spliteval_->AddSplit(nid,
-                               cleft,
-                               cright,
-                               snode_[nid].best.SplitIndex(),
-                               snode_[cleft].weight,
-                               snode_[cright].weight);
+
+          tree_evaluator_.AddSplit(nid, cleft, cright, snode_[nid].best.SplitIndex(),
+                                   snode_[cleft].weight, snode_[cright].weight);
           interaction_constraints_.Split(nid, snode_[nid].best.SplitIndex(), cleft, cright);
         }
         qexpand_ = newnodes;
@@ -189,8 +200,7 @@ class ColMaker: public TreeUpdater {
    protected:
     // initialize temp data structure
     inline void InitData(const std::vector<GradientPair>& gpair,
-                         const DMatrix& fmat,
-                         const RegTree& tree) {
+                         const DMatrix& fmat) {
       {
         // setup position
         position_.resize(gpair.size());
@@ -202,6 +212,9 @@ class ColMaker: public TreeUpdater {
         }
         // mark subsample
         if (param_.subsample < 1.0f) {
+          CHECK_EQ(param_.sampling_method, TrainParam::kUniform)
+            << "Only uniform sampling is supported, "
+            << "gradient-based sampling is only support by GPU Hist.";
           std::bernoulli_distribution coin_flip(param_.subsample);
           auto& rnd = common::GlobalRandom();
           for (size_t ridx = 0; ridx < position_.size(); ++ridx) {
@@ -211,8 +224,10 @@ class ColMaker: public TreeUpdater {
         }
       }
       {
-        column_sampler_.Init(fmat.Info().num_col_, param_.colsample_bynode,
-                             param_.colsample_bylevel, param_.colsample_bytree);
+        column_sampler_.Init(fmat.Info().num_col_,
+                             fmat.Info().feature_weigths.ConstHostVector(),
+                             param_.colsample_bynode, param_.colsample_bylevel,
+                             param_.colsample_bytree);
       }
       {
         // setup temp space for each thread
@@ -263,13 +278,15 @@ class ColMaker: public TreeUpdater {
         // update node statistics
         snode_[nid].stats = stats;
       }
+
+      auto evaluator = tree_evaluator_.GetEvaluator();
       // calculating the weights
       for (int nid : qexpand) {
-        bst_uint parentid = tree[nid].Parent();
+        bst_node_t parentid = tree[nid].Parent();
         snode_[nid].weight = static_cast<float>(
-            spliteval_->ComputeWeight(parentid, snode_[nid].stats));
+            evaluator.CalcWeight(parentid, param_, snode_[nid].stats));
         snode_[nid].root_gain = static_cast<float>(
-            spliteval_->ComputeScore(parentid, snode_[nid].stats, snode_[nid].weight));
+            evaluator.CalcGain(parentid, param_, snode_[nid].stats));
       }
     }
     /*! \brief update queue expand add in new leaves */
@@ -286,9 +303,10 @@ class ColMaker: public TreeUpdater {
     }
 
     // update enumeration solution
-    inline void UpdateEnumeration(int nid, GradientPair gstats,
-                                  bst_float fvalue, int d_step, bst_uint fid,
-                                  GradStats &c, std::vector<ThreadEntry> &temp) const { // NOLINT(*)
+    inline void UpdateEnumeration(
+        int nid, GradientPair gstats, bst_float fvalue, int d_step,
+        bst_uint fid, GradStats &c, std::vector<ThreadEntry> &temp, // NOLINT(*)
+        TreeEvaluator::SplitEvaluator<TrainParam> const &evaluator) const {
       // get the statistics of nid
       ThreadEntry &e = temp[nid];
       // test if first hit, this is fine, because we set 0 during init
@@ -304,7 +322,7 @@ class ColMaker: public TreeUpdater {
             bst_float loss_chg {0};
             if (d_step == -1) {
               loss_chg = static_cast<bst_float>(
-                  spliteval_->ComputeSplitScore(nid, fid, c, e.stats) -
+                  evaluator.CalcSplitGain(param_, nid, fid, c, e.stats) -
                   snode_[nid].root_gain);
               bst_float proposed_split = (fvalue + e.last_fvalue) * 0.5f;
               if ( proposed_split == fvalue ) {
@@ -316,7 +334,7 @@ class ColMaker: public TreeUpdater {
               }
             } else {
               loss_chg = static_cast<bst_float>(
-                  spliteval_->ComputeSplitScore(nid, fid, e.stats, c) -
+                  evaluator.CalcSplitGain(param_, nid, fid, e.stats, c) -
                   snode_[nid].root_gain);
               bst_float proposed_split = (fvalue + e.last_fvalue) * 0.5f;
               if ( proposed_split == fvalue ) {
@@ -335,12 +353,11 @@ class ColMaker: public TreeUpdater {
       }
     }
     // same as EnumerateSplit, with cacheline prefetch optimization
-    void EnumerateSplit(const Entry *begin,
-                        const Entry *end,
-                        int d_step,
-                        bst_uint fid,
-                        const std::vector<GradientPair> &gpair,
-                        std::vector<ThreadEntry> &temp)  const { // NOLINT(*)
+    void EnumerateSplit(
+        const Entry *begin, const Entry *end, int d_step, bst_uint fid,
+        const std::vector<GradientPair> &gpair,
+        std::vector<ThreadEntry> &temp, // NOLINT(*)
+        TreeEvaluator::SplitEvaluator<TrainParam> const &evaluator) const {
       CHECK(param_.cache_opt) << "Support for `cache_opt' is removed in 1.0.0";
       const std::vector<int> &qexpand = qexpand_;
       // clear all the temp statistics
@@ -375,7 +392,7 @@ class ColMaker: public TreeUpdater {
           if (nid < 0 || !interaction_constraints_.Query(nid, fid)) { continue; }
           this->UpdateEnumeration(nid, buf_gpair[i],
                                   p->fvalue, d_step,
-                                  fid, c, temp);
+                                  fid, c, temp, evaluator);
         }
       }
 
@@ -389,7 +406,7 @@ class ColMaker: public TreeUpdater {
         if (nid < 0 || !interaction_constraints_.Query(nid, fid)) { continue; }
         this->UpdateEnumeration(nid, buf_gpair[i],
                                 it->fvalue, d_step,
-                                fid, c, temp);
+                                fid, c, temp, evaluator);
       }
       // finish updating all statistics, check if it is possible to include all sum statistics
       for (int nid : qexpand) {
@@ -402,13 +419,13 @@ class ColMaker: public TreeUpdater {
           const bst_float delta = d_step == +1 ? gap: -gap;
           if (d_step == -1) {
             loss_chg = static_cast<bst_float>(
-                spliteval_->ComputeSplitScore(nid, fid, c, e.stats) -
+                evaluator.CalcSplitGain(param_, nid, fid, c, e.stats) -
                 snode_[nid].root_gain);
             e.best.Update(loss_chg, fid, e.last_fvalue + delta, d_step == -1, c,
                           e.stats);
           } else {
             loss_chg = static_cast<bst_float>(
-                spliteval_->ComputeSplitScore(nid, fid, e.stats, c) -
+                evaluator.CalcSplitGain(param_, nid, fid, e.stats, c) -
                 snode_[nid].root_gain);
             e.best.Update(loss_chg, fid, e.last_fvalue + delta, d_step == -1,
                           e.stats, c);
@@ -421,39 +438,37 @@ class ColMaker: public TreeUpdater {
     virtual void UpdateSolution(const SparsePage &batch,
                                 const std::vector<bst_feature_t> &feat_set,
                                 const std::vector<GradientPair> &gpair,
-                                DMatrix*p_fmat) {
+                                DMatrix*) {
       // start enumeration
       const auto num_features = static_cast<bst_omp_uint>(feat_set.size());
 #if defined(_OPENMP)
       const int batch_size =  // NOLINT
           std::max(static_cast<int>(num_features / this->nthread_ / 32), 1);
 #endif  // defined(_OPENMP)
-
       {
-        std::vector<float> densities(num_features);
-        CHECK_EQ(feat_set.size(), num_features);
-        for (bst_omp_uint i = 0; i < num_features; ++i) {
-          bst_feature_t const fid = feat_set[i];
-          densities.at(i) = p_fmat->GetColDensity(fid);
-        }
-
+        dmlc::OMPException omp_handler;
 #pragma omp parallel for schedule(dynamic, batch_size)
         for (bst_omp_uint i = 0; i < num_features; ++i) {
-          bst_feature_t const fid = feat_set[i];
-          int32_t const tid = omp_get_thread_num();
-          auto c = batch[fid];
-          const bool ind = c.size() != 0 && c[0].fvalue == c[c.size() - 1].fvalue;
-          auto const density = densities[i];
-          if (colmaker_train_param_.NeedForwardSearch(
-                  param_.default_direction, density, ind)) {
-            this->EnumerateSplit(c.data(), c.data() + c.size(), +1,
-                                 fid, gpair, stemp_[tid]);
-          }
-          if (colmaker_train_param_.NeedBackwardSearch(param_.default_direction)) {
-            this->EnumerateSplit(c.data() + c.size() - 1, c.data() - 1, -1,
-                                 fid, gpair, stemp_[tid]);
-          }
+          omp_handler.Run([&]() {
+            auto evaluator = tree_evaluator_.GetEvaluator();
+            bst_feature_t const fid = feat_set[i];
+            int32_t const tid = omp_get_thread_num();
+            auto c = batch[fid];
+            const bool ind =
+                c.size() != 0 && c[0].fvalue == c[c.size() - 1].fvalue;
+            if (colmaker_train_param_.NeedForwardSearch(
+                    param_.default_direction, column_densities_[fid], ind)) {
+              this->EnumerateSplit(c.data(), c.data() + c.size(), +1, fid,
+                                   gpair, stemp_[tid], evaluator);
+            }
+            if (colmaker_train_param_.NeedBackwardSearch(
+                    param_.default_direction)) {
+              this->EnumerateSplit(c.data() + c.size() - 1, c.data() - 1, -1,
+                                   fid, gpair, stemp_[tid], evaluator);
+            }
+          });
         }
+        omp_handler.Rethrow();
       }
     }
     // find splits at current level, do split per level
@@ -462,6 +477,8 @@ class ColMaker: public TreeUpdater {
                           const std::vector<GradientPair> &gpair,
                           DMatrix *p_fmat,
                           RegTree *p_tree) {
+      auto evaluator = tree_evaluator_.GetEvaluator();
+
       auto feat_set = column_sampler_.GetFeatureSet(depth);
       for (const auto &batch : p_fmat->GetBatches<SortedCSCPage>()) {
         this->UpdateSolution(batch, feat_set->HostVector(), gpair, p_fmat);
@@ -474,15 +491,17 @@ class ColMaker: public TreeUpdater {
         // now we know the solution in snode[nid], set split
         if (e.best.loss_chg > kRtEps) {
           bst_float left_leaf_weight =
-              spliteval_->ComputeWeight(nid, e.best.left_sum) *
+              evaluator.CalcWeight(nid, param_, e.best.left_sum) *
               param_.learning_rate;
           bst_float right_leaf_weight =
-              spliteval_->ComputeWeight(nid, e.best.right_sum) *
+              evaluator.CalcWeight(nid, param_, e.best.right_sum) *
               param_.learning_rate;
           p_tree->ExpandNode(nid, e.best.SplitIndex(), e.best.split_value,
                              e.best.DefaultLeft(), e.weight, left_leaf_weight,
                              right_leaf_weight, e.best.loss_chg,
-                             e.stats.sum_hess, 0);
+                             e.stats.sum_hess,
+                             e.best.left_sum.GetHess(), e.best.right_sum.GetHess(),
+                             0);
         } else {
           (*p_tree)[nid].SetLeaf(e.weight * param_.learning_rate);
         }
@@ -591,175 +610,17 @@ class ColMaker: public TreeUpdater {
     std::vector<NodeEntry> snode_;
     /*! \brief queue of nodes to be expanded */
     std::vector<int> qexpand_;
-    // Evaluates splits and computes optimal weights for a given split
-    std::unique_ptr<SplitEvaluator> spliteval_;
+    TreeEvaluator tree_evaluator_;
 
     FeatureInteractionConstraintHost interaction_constraints_;
+    const std::vector<float> &column_densities_;
   };
 };
 
-// distributed column maker
-class DistColMaker : public ColMaker {
- public:
-  void Configure(const Args& args) override {
-    param_.UpdateAllowUnknown(args);
-    pruner_.reset(TreeUpdater::Create("prune", tparam_));
-    pruner_->Configure(args);
-    spliteval_.reset(SplitEvaluator::Create(param_.split_evaluator));
-    spliteval_->Init(&param_);
-  }
-
-  char const* Name() const override {
-    return "distcol";
-  }
-
-  void Update(HostDeviceVector<GradientPair> *gpair,
-              DMatrix* dmat,
-              const std::vector<RegTree*> &trees) override {
-    CHECK_EQ(trees.size(), 1U) << "DistColMaker: only support one tree at a time";
-    Builder builder(
-      param_,
-      colmaker_param_,
-      std::unique_ptr<SplitEvaluator>(spliteval_->GetHostClone()),
-      interaction_constraints_);
-    // build the tree
-    builder.Update(gpair->ConstHostVector(), dmat, trees[0]);
-    //// prune the tree, note that pruner will sync the tree
-    pruner_->Update(gpair, dmat, trees);
-    // update position after the tree is pruned
-    builder.UpdatePosition(dmat, *trees[0]);
-  }
-
- private:
-  class Builder : public ColMaker::Builder {
-   public:
-    explicit Builder(const TrainParam &param,
-                     ColMakerTrainParam const& colmaker_train_param,
-                     std::unique_ptr<SplitEvaluator> spliteval,
-                     FeatureInteractionConstraintHost _interaction_constraints)
-        : ColMaker::Builder(param, colmaker_train_param,
-                            std::move(spliteval),
-                            std::move(_interaction_constraints)) {}
-    inline void UpdatePosition(DMatrix* p_fmat, const RegTree &tree) {
-      const auto ndata = static_cast<bst_omp_uint>(p_fmat->Info().num_row_);
-      #pragma omp parallel for schedule(static)
-      for (bst_omp_uint ridx = 0; ridx < ndata; ++ridx) {
-        int nid = this->DecodePosition(ridx);
-        while (tree[nid].IsDeleted()) {
-          nid = tree[nid].Parent();
-          CHECK_GE(nid, 0);
-        }
-        this->position_[ridx] = nid;
-      }
-    }
-
-   protected:
-    void SetNonDefaultPosition(const std::vector<int> &qexpand, DMatrix *p_fmat,
-                               const RegTree &tree) override {
-      // step 2, classify the non-default data into right places
-      std::vector<unsigned> fsplits;
-      for (int nid : qexpand) {
-        if (!tree[nid].IsLeaf()) {
-          fsplits.push_back(tree[nid].SplitIndex());
-        }
-      }
-      // get the candidate split index
-      std::sort(fsplits.begin(), fsplits.end());
-      fsplits.resize(std::unique(fsplits.begin(), fsplits.end()) - fsplits.begin());
-      while (fsplits.size() != 0 && fsplits.back() >= p_fmat->Info().num_col_) {
-        fsplits.pop_back();
-      }
-      // bitmap is only word concurrent, set to bool first
-      {
-        auto ndata = static_cast<bst_omp_uint>(this->position_.size());
-        boolmap_.resize(ndata);
-        #pragma omp parallel for schedule(static)
-        for (bst_omp_uint j = 0; j < ndata; ++j) {
-            boolmap_[j] = 0;
-        }
-      }
-      for (const auto &batch : p_fmat->GetBatches<SortedCSCPage>()) {
-        for (auto fid : fsplits) {
-          auto col = batch[fid];
-          const auto ndata = static_cast<bst_omp_uint>(col.size());
-          #pragma omp parallel for schedule(static)
-          for (bst_omp_uint j = 0; j < ndata; ++j) {
-            const bst_uint ridx = col[j].index;
-            const bst_float fvalue = col[j].fvalue;
-            const int nid = this->DecodePosition(ridx);
-            if (!tree[nid].IsLeaf() && tree[nid].SplitIndex() == fid) {
-              if (fvalue < tree[nid].SplitCond()) {
-                if (!tree[nid].DefaultLeft()) boolmap_[ridx] = 1;
-              } else {
-                if (tree[nid].DefaultLeft()) boolmap_[ridx] = 1;
-              }
-            }
-          }
-        }
-      }
-
-      bitmap_.InitFromBool(boolmap_);
-      // communicate bitmap
-      rabit::Allreduce<rabit::op::BitOR>(dmlc::BeginPtr(bitmap_.data), bitmap_.data.size());
-      // get the new position
-      const auto ndata = static_cast<bst_omp_uint>(p_fmat->Info().num_row_);
-      #pragma omp parallel for schedule(static)
-      for (bst_omp_uint ridx = 0; ridx < ndata; ++ridx) {
-        const int nid = this->DecodePosition(ridx);
-        if (bitmap_.Get(ridx)) {
-          CHECK(!tree[nid].IsLeaf()) << "inconsistent reduce information";
-          if (tree[nid].DefaultLeft()) {
-            this->SetEncodePosition(ridx, tree[nid].RightChild());
-          } else {
-            this->SetEncodePosition(ridx, tree[nid].LeftChild());
-          }
-        }
-      }
-    }
-    // synchronize the best solution of each node
-    void SyncBestSolution(const std::vector<int> &qexpand) override {
-      std::vector<SplitEntry> vec;
-      for (int nid : qexpand) {
-        for (int tid = 0; tid < this->nthread_; ++tid) {
-          this->snode_[nid].best.Update(this->stemp_[tid][nid].best);
-        }
-        vec.push_back(this->snode_[nid].best);
-      }
-      // TODO(tqchen) lazy version
-      // communicate best solution
-      reducer_.Allreduce(dmlc::BeginPtr(vec), vec.size());
-      // assign solution back
-      for (size_t i = 0; i < qexpand.size(); ++i) {
-        const int nid = qexpand[i];
-        this->snode_[nid].best = vec[i];
-      }
-    }
-
-   private:
-    common::BitMap bitmap_;
-    std::vector<int> boolmap_;
-    rabit::Reducer<SplitEntry, SplitEntry::Reduce> reducer_;
-  };
-  // we directly introduce pruner here
-  std::unique_ptr<TreeUpdater> pruner_;
-  // training parameter
-  TrainParam param_;
-  // Cloned for each builder instantiation
-  std::unique_ptr<SplitEvaluator> spliteval_;
-
-  FeatureInteractionConstraintHost interaction_constraints_;
-};
-
 XGBOOST_REGISTER_TREE_UPDATER(ColMaker, "grow_colmaker")
 .describe("Grow tree with parallelization over columns.")
 .set_body([]() {
     return new ColMaker();
   });
-
-XGBOOST_REGISTER_TREE_UPDATER(DistColMaker, "distcol")
-.describe("Distributed column split version of tree maker.")
-.set_body([]() {
-    return new DistColMaker();
-  });
 }  // namespace tree
 }  // namespace xgboost
